# V486.tcl---CAEN V486 8 Ch gate & delay module control frame

set frame "V486"
set frametitle "CAEN V486 8 Channel Gate & Delay generator"
set icontitle "V486"

set modulename v486
set moduletype  "CAEN V486"

set moduleindex 0

set maxmodule [inform vmecontrol inquire [set modulename]count]
if {$maxmodule > 0} {
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}
} else {
set modulelist {99}
}

set module [lindex $modulelist $moduleindex]
set adc  0

set maxadc 8

set alladcs 0
set allmodules 0


for {set i 0} {$i < $maxadc} {incr i} {
   set w2r(DELAY$i)   Delay$i
   set adc2r(DELAY$i) Delay\[?,??\]
   set w2r(GATE$i)     Gate$i
   set adc2r(GATE$i)   Gate\[?,??\]
}
set w2r(MODE)           Mode
set w2r(MUX)            Mux
set w2r(CLEAR)          Clear
set w2r(TEST)           Test


loadsource common.vme

proc click-widget {w value} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc click-adc-widget {w value} {
    global w2r adc2r
    global modulename module
    global alladcs allmodules

    if {$alladcs == 0} then {set register $w2r($w)} else {set register $adc2r($w)}

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc click-select-module {v} {
   global module maxmodule
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-all-adcs {v} {global alladcs; set alladcs $v}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc click-ID {} {
    global modulename module
    foreach id {1 2 3} {
       set vl [read-vme-register $modulename#$module.ModID#$id]
       insert-log "V486 Module Identifier Word $id = $vl"
    }
}
   
proc redisplay-module {} {
   global w2r modulename module maxmodule

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   foreach w {MODE MUX} {
       set register $w2r($w)
       set value [read-vme-register $modulename#$module.$register]
       :modify $w :value $value
   }
}

proc redisplay-adcs {} {
    global w2r
    global modulename module maxadc

    foreach W {DELAY GATE} {
       for {set i 0} {$i < $maxadc} {incr i} {
          set w $W$i
          set register $w2r($w)
          set value [read-vme-register $modulename#$module.$register]
          :modify $w :value $value
       }
    }
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module
   redisplay-adcs

   clear-busy
   set EGreport 0
}

proc start args {
   global started EGreport modulename
   
   if {$started == 0} then {
       make-frame
       EGbindings
   }
   set started 1
   open-frame

   clear-footer
   clear-busy
   set EGreport 0

   loadsource [set modulename].defn

   click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}


proc click arg {
   global debug
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-ADCS    {click-all-adcs $v}

      MUX  -
      MODE      -
      CLEAR     -
      TEST        {click-widget $w $v}
      
      ID          {click-ID}

      DELAY*     -
      GATE*       {click-adc-widget $w $v}

      REDISPLAY      {click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env ac
  global module maxmodule modulelist maxadc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :col-gap 30
     :checkbox ALL-MODULES :label "Act on ALL modules?"
     :checkbox ALL-ADCS    :label "Act on ALL Channels?"

     :next-row T
     :message M1 :label "Module Setup" :bold T
     :next-row T
     :col-gap 30

     :choice MUX    :label "Multiplex Output"    :strings [list 0 1 2 3 4 5 6 7]
     :choice MODE   :label "Mode"   :strings [list normal test couple test]

   :panel ADC :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 0
     
     :message M20 :label "Channel Gate" :bold T
     :next-row T

     for {set i 0} {$i < $maxadc} {} {
         :slider   GATE$i  :label "$i" :min 0 :max 255 :entry-width 3
         :hskip 10
         incr i
         if {[expr ($i / 4) * 4] == $i} {:next-row T}
     }
     
     :message M21 :label "Channel Delay" :bold T
     :next-row T
     
     for {set i 0} {$i < $maxadc} {} {
         :slider   DELAY$i  :label "$i" :min 0 :max 255 :entry-width 3
         :hskip 10
         incr i
         if {[expr ($i / 4) * 4] == $i} {:next-row T}
     }
  
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :button CLEAR :label "Clear"
      :button TEST :label "Generate Test Strobe"
      :button ID   :label "Identify Module"

  :show T

}
