# vmecontrol.tcl---VME Configuration & Control Frame
#    common section 


set debug 0
set started 0

set frame VME
set frametitle "VME Configuration & Control Frame"
set icontitle "VME"

if {[info exists CurrentVMEServer] == 0} then {

   if {[info exists env(VmeServer)] == 0} then {set CurrentVMEServer vme}  \
      else {set CurrentVMEServer $env(VmeServer)}
}

set VMEServerList {vme1}
set VMEModuleList(vme1) {}
set RoleList [list VME vme]
set Role vme

set Unknown VME

#    load moduledefinition list

loadsource vmecontrol.defn

set modulechoice {}
set modulenames  {}
set tasknames    {}

foreach item $moduledefinition {
    if {[first $item] != ""} then {
        set [second $item]count 0
        append modulechoice " " \"[first $item]\"
        append modulenames  " " [second $item]
        append tasknames    " " [first [third $item]]
    }
}

set moduletype 	[first $modulechoice]
set modulename	[first $modulenames]

set A16Address 0x0000
set A24Address 0x000000
set A32Address 0x00000000
set Ilevel 0
set Ivector 0

set vmeserver ""
set vmeaddress 0

proc make_vmeaddress {} {
   global A16Address A24Address A32Address Ilevel Ivector
   global vmeaddress
   set vmeaddress "$A16Address:$A24Address:$A32Address:$Ilevel:$Ivector"
}

proc make_vmeserver {} {
   global CurrentVMEServer
   global vmeserver
   set vmeserver $CurrentVMEServer
}

make_vmeaddress
make_vmeserver
   
set iindex   1
set vsn      1
set subaddr  0
set numchans 0
set group    0
set item     0
set ipg      0
set iincr    1
set icomment ""

proc eleventh p {lindex $p 10}

set occupied {}
set datalist {}

set Configdir "$env(MIDASEXPBASE)"
catch {set Configdir "$env(EXPERIMENTBASE)"}

set LUTdir "$env(MIDASEXPBASE)"
catch {set LUTdir "$env(EXPERIMENTBASE)"}
set LUTfile "LUT.vme"
set LUTmsg "\n Do you wish to create a new LUT file?\n\
       You will need to do this if you have changed the configuration of VME data sources\n\n"

set VATdir $LUTdir
set VATfile "VAT.vme"

set LUTmsg "\n Do you wish to create new LUT and VAT files?\n\
       You will need to do this if you have changed the configuration of VME data sources\n\n"

set selection -1

set m {
    {{Module} {Module} {} {    Address} {Module} {Channel} {Number of} {Group } {Item  } {Items/} {Item} {}       }
    {{Type  } {Name  } {} {Information} {ID    } {ID     } {channels } {Number} {Number} {group } {Incr} {Comment}}
      }

#    configuration list 
#      for each module need 
#        1.   module type
#        2.   vme server
#        3.   vme address info
#        4.   module ID
#        5.   ID of first channel  (channel number)
#        6.   number of channels
#        7.   group number for first channel
#        8.   item number for first channel
#        9.   number of items per group
#        10.  item increment per channel
#        11.   comment

if {$LUTpresent != 0} {
set defaultlist {                                                             \
                 {{CAEN V785} {} vme 0x0000:0x10000000:0x00000000:0:0 1 0 32 100 0 4 1 "Adc #1"} \
                 {{CAEN V785} {} vme 0x0000:0x18000000:0x00000000:0:0 2 0 32 108 0 4 1 "Adc #2"} \
                }
} else {
set defaultlist {                                                             \
                 {{CAEN V785} {} vme 0x0000:0x10000000:0x00000000:0:0 1 "Adc #1"} \
                 {{CAEN V785} {} vme 0x0000:0x18000000:0x00000000:0:0 2 "Adc #2"} \
                }
}

proc SelectServer {s} {
    global CurrentVMEServer VMEServerList
    set CurrentVMEServer $s
    :modify VMESERVER :inactive F :strings $VMEServerList :value [lsearch $VMEServerList $CurrentVMEServer]
    if {[llength $VMEServerList] <= 1} then {:modify VMESERVER :inactive T}
    make_vmeserver
    restore_configuration
}

#    obtain a list of the available VME servers from the experiment file

proc make-serverlist {} {
  global VMEServerList RoleList VMEModuleList Role

#    find VME crates available

   set VMECrateList ""
   if {[catch {eg "(experiment ?resources)"} m]} then {
       midas-warning "no experiment connected!!!"
   } else {
       foreach role $RoleList {
          set c [inform ex obtain_RPCServer_byRole $role]
          foreach item $c {
              append VMECrateList " [first [inform ex obtain_RPCServerInfo $item $role]]"
              set Role $role
          }

          set c [inform ex obtain_SOAPServer_byRole $role]
          foreach item $c {
              append VMECrateList " [first [inform ex obtain_SOAPServerInfo $item $role]]"
              set Role $role
          }
       }
   }
#    sort lists into sensible order
   set serverlist [lsort $VMECrateList]
   if {[llength $serverlist] == 0} then {
      midas-warning "no VME crates found in current experiment!!!"
      set VMEServerList {vme}
      set VMEModuleList(vme) {}
   } else {
      set VMEServerList {}
      foreach server $serverlist {
          lappend VMEServerList $server
          set VMEModuleList($server) {}
      }
   }
}

proc VMEmodule2server {requested} {
  global Unknown
  global VMEServerList VMEModuleList
 
    foreach server $VMEServerList {
       set modulelist $VMEModuleList($server)
       if {[lsearch $modulelist $requested] != -1} then {
#          found the VME crate we are looking for -  return server name to access this VME crate
         return $server
       }
    }
   return $Unknown
}


proc initialise {} {
    global moduledefinition

#    load module definitions and action procedures
   foreach litem $moduledefinition {
      if {[first $litem] != ""} then {
         set lmoduletype [first $litem]
         set ltaskname   [first [third $litem]]
         catch {loadsource $ltaskname.defn}
         set lname "_registersavepatterns"
         global $lmoduletype$lname
         if {[info exists $lmoduletype$lname] == 0} then {set $lmoduletype$lname {}}
      }
   }
}

proc set-experiment {dir} {global mytaskname; inform bf restart $mytaskname}

proc {bus-error} {module} {

   display-error-report self "Bus error accessing $module - module probably missing or broken" \
        "check server and address info supplied in configuration are correct"
}

proc DAQ-action {procname} {
   global env debug datalist occupied VMEServerList CurrentVMEServer
   if {[catch {set x $env(EXPERIMENTBASE)}] != 0} then {midas-warning "No Current Experiment!"; return}
   set MyServer $CurrentVMEServer
   foreach server $VMEServerList {
      SelectServer $server
      set selection 0
      foreach data $datalist {
         set moduletype [first $data]
         set modulename [second $data]
         set vmeserver [third $data]
         set vmeaddress [fourth $data]
         set vsn [fifth $data]
         if {[set occupied(selection,$vmeserver,$vmeaddress)] == $selection} then {
#    first occurence of this specific module in the configuration
             if {$debug} then {insert-debug "$procname $moduletype $modulename"}
             if {[info procs $moduletype$procname] != ""} then {$moduletype$procname $modulename $vmeserver $vsn}
         }
         incr selection
      }
   }
   if {$MyServer != $CurrentVMEServer} then {SelectServer $MyServer}
}

proc DAQ-reset {} {DAQ-action "_DAQ-reset"}
proc DAQ-go {}    {DAQ-action "_DAQ-go"}
proc DAQ-halt {}  {DAQ-action "_DAQ-halt"}
proc DAQ-setup {} {DAQ-action "_DAQ-setup"}
proc DAQ-flush {} {DAQ-action "_DAQ-flush"}


proc read-vmestate {} {return -1}

if {$LUTpresent != 0} {

proc format-data l {
  set ll {}
  foreach f $l {lappend ll [format "%12s %8s %32s %8s %8s %8s %8s %6s %6s %6s %24s" \
   [first $f] [second $f] [fourth $f] [fifth $f] [sixth $f] [seventh $f] [eigth $f] [nineth $f] [tenth $f] [eleventh $f] [lindex $f 11]]}
  return $ll
}
} else {

proc format-data l {
  set ll {}
  foreach f $l {lappend ll [format "%12s %8s %32s %8s %24s" \
   [first $f] [second $f] [fourth $f] [fifth $f] [lindex $f 11]]}
  return $ll
}
}

proc make-datalist {} {
   global moduledefinition modulechoice modulenames
   global datalist
   global VMEModuleList VMEServerList CurrentVMEServer

   set VMEModuleList($CurrentVMEServer) {}

#    delete the table occupied and recreate as a way of deleting all elements
   global occupied; catch {unset occupied}; global occupied
   set selection 0

   foreach data $datalist {
#      load-data $selection

      set moduletype "[first $data]"
      set vmeserver [third $data]
      set vmeaddress [fourth $data]
      set vsn [fifth $data]
      set subaddr [sixth $data]
      set numchans [seventh $data]
      set group [eigth $data]
      set item [nineth $data]
      set ipg [tenth $data]
      set iincr [eleventh $data]
      set icom "[lindex $data 11]"
      set iindex [lindex $data 12]
      if {$iindex == ""} {set iindex $vsn}
      set moduleindex [lsearch $modulechoice $moduletype]
      set modulename [lindex $modulenames $moduleindex]

      if {[info exists occupied(name,$vmeserver,$vmeaddress)] == 0} then {
#      first use of VME crate slot
         if {$moduleindex >= 0} {
            append modulename  #  $iindex
            set occupied(name,$vmeserver,$vmeaddress) [lindex $modulenames $moduleindex]
            set occupied(selection,$vmeserver,$vmeaddress) $selection
            lappend VMEModuleList($vmeserver) $modulename
         } else {
            midas-warning "module $moduletype is in the configuration file but is not a defined module for this experiment"
         }
      } else {
#      VME crate slot already mentioned
         set modulename [set occupied(name,$vmeserver,$vmeaddress)]
         if {[lindex $modulenames $moduleindex] == $modulename} then {
#          same module type is probably OK
            set-footer "Warning - VME server $vmeserver address $vmeaddress previously mentioned"
            append modulename #  $iindex
         } else {
            display-error-report self "VME $vmeserver address $vmeaddress used by $modulename and [lindex $modulenames $moduleindex]"
         }
      }

      set datalist [lreplace $datalist $selection $selection \
         [list $moduletype $modulename $vmeserver $vmeaddress $vsn $subaddr $numchans $group $item $ipg $iincr $icom $iindex]]
      incr selection
   }

#     this computes the module name instance count for each module

   foreach item $moduledefinition {
      if {[first $item] != ""} then {
        global [second $item]count
        set [second $item]count 0
      }
   }

   foreach server $VMEServerList {
       foreach module $VMEModuleList($server) {
          set moduletype [first [split $module #]]
          incr [set moduletype]count
       }
   }

   paint-datalist
   repaint
}

proc makeLUT {} {
  global env debug datalist LUTdir LUTfile

  set-footer "Making the LUT file"

  set z [catch {open [file join $LUTdir $LUTfile] w} file]
  if {$z != 0} then {set-footer "$file"; return}

  puts $file "# VME GEO address +  channel (adc) => group/item conversion"
  puts $file "# VME Module: GEO Address  :   channel  :  group   :  item "

#    calculate LUT entry for each adc in each module
  foreach data $datalist {
    set moduletype [first $data]
    if {$moduletype == "CAEN V785" || $moduletype == "CAEN V785N" || $moduletype == "CAEN V775" \
        || $moduletype == "CAEN V792" || $moduletype == "CAEN V879" || $moduletype == "CAEN V878"} then {
       set moduletype "CAEN V785"
    }
    set vsn [fifth $data]
    set subaddr [sixth $data]
    set numchans [seventh $data]
    set group [eigth $data]
    set item [nineth $data]
    set ipg [tenth $data]
    set iincr [eleventh $data]

    set items 0
    for {set i 0} {$i < $numchans} {incr i} {
      incr items
      puts $file "$moduletype : $vsn : $subaddr : $group : $item"
      incr subaddr
      incr item $iincr
      if {$items >= $ipg} then {set item [nineth $data]; incr group; set items 0}
    }
  }

  close $file
  set-footer "LUT written to [file join $LUTdir $LUTfile]"
}

proc makeVAT {} {
  global env debug datalist VATdir VATfile

  set-footer "Making the VAT file"

  set z [catch {open [file join $VATdir $VATfile] w} file]
  if {$z != 0} then {set-footer "$file"; return}

#    calculate VAT entry for each module
  foreach data $datalist {
    set moduletype [first $data]
    if {$moduletype == "CAEN V785" || $moduletype == "CAEN V785N" || $moduletype == "CAEN V775" \
        || $moduletype == "CAEN V792" || $moduletype == "CAEN V879" || $moduletype == "CAEN V878"} then {
       set module [second $data]
       set vmeserver [third $data]
       set vmeaddress [fourth $data]
       set a32addr($module) [third [split $vmeaddress :]]
    }
  }
  foreach item [array names a32addr] {puts $file $a32addr($item)}

  close $file
  set-footer "VAT written to [file join $VATdir $VATfile]"
}

proc load-data n {
   global datalist moduletype modulename moduleindex vmeserver vmeaddress vsn subaddr numchans group item ipg iincr icomment iindex

   set data [lindex $datalist $n]

      set moduletype "[first $data]"
      set modulename [second $data]
      set vmeserver [third $data]
      set vmeaddress [fourth $data]
      set vsn [fifth $data]
      set subaddr [sixth $data]
      set numchans [seventh $data]
      set group [eigth $data]
      set item [nineth $data]
      set ipg [tenth $data]
      set iincr [eleventh $data]
      set icomment "[lindex $data 11]"
      set iindex [lindex $data 12]
}

proc load-configuration {} {
   global debug env EGreport CurrentVMEServer Configdir
   global datalist occupied modulenames

#    load hardware configuration into the register server

   set-footer "deleting all VME registers"
   foreach module $modulenames {
     EG  "register :pattern '/$CurrentVMEServer/$module*' !delete"
   }

   set-footer "Creating Configuration"
   set EGreport -1
   set-busy

   set selection 0
   foreach data $datalist {
     set moduletype [first $data]
     set modulename [second $data]
     set vmeserver [third $data]
     set vmeaddress [fourth $data]
     set vsn [fifth $data]
     if {[set occupied(selection,$vmeserver,$vmeaddress)] == $selection} then {
#    first occurence of this specific module in the configuration
        set-footer "Creating module $modulename"
        if {[info procs Add_[set moduletype]_Module] != ""} then {Add_[set moduletype]_Module $modulename $vmeserver $vmeaddress}
        set-footer "Initialising module $modulename"
        if {[info procs [set moduletype]_DAQ-setup] != ""} then {[set moduletype]_DAQ-setup $modulename $vmeserver $vsn}
     }
     incr selection
   }

   clear-busy
   set EGreport 0

#    write the current configuration to disc

   set z [catch {open [file join $Configdir [set CurrentVMEServer]_current_configuration] w} file]
   if {$z != 0} then {set-footer "$file"; return}
   foreach data $datalist {puts $file "$data"}
   close $file

#    record that we have done it

    write-vme-register NetVar_HasBeenConfigured 1

   clear-footer
}

proc load-configuration+LUT {} {
    global LUTmsg

    load-configuration
    if {[midas-confirmation "$LUTmsg"] != 0} then {makeLUT; makeVAT}
}

proc HasBeenConfigured {} {
   global VMEServerList

   set vl 0
   foreach server $VMEServerList {
      SelectServer $server
      incr vl [read-vme-register NetVar_HasBeenConfigured]
   }
   return [expr $vl / [llength $VMEServerList]]
}

proc restore_configuration {} {
   global env Configdir CurrentVMEServer datalist

   set datalist ""
   make-datalist

   set z [catch {open [file join $Configdir [set CurrentVMEServer]_current_configuration] r} file]
   if {$z != 0} then {set-footer "$file"; return}

   set configuration [split [read $file] \n]
   close $file
   set l [expr [llength $configuration] - 1]
   set datalist [lreplace $configuration $l $l]
   make-datalist
}

proc Configuration {} {
   global VMEServerList datalist

   set config {}
   foreach server $VMEServerList {
      SelectServer $server
      lappend config $datalist
   }
   return $config
}

proc RestoreConfiguration {l} {
    global CurrentVMEServer datalist

    set datalist {}

    foreach item $l {
        set server [third $item]
        if {$server == $CurrentVMEServer} then {lappend datalist $item}
    }

    make-datalist
}

proc Configure {} {
   global VMEServerList datalist

   foreach server $VMEServerList {
      SelectServer $server
      load-configuration
   }
}


proc paint-datalist {} {
  global env datalist selection
  global m
  set pointstrings [format-data $datalist]
  set strings [format-data $m]
    :modify M :empty T :strings "$strings"
    :modify CONFIG-DATALIST :show F :empty T :strings "$pointstrings" :show T
    :modify CONFIG-DATALIST :select $selection
    :modify CONFIG-DATALIST :see 0
}

proc repaint {} {
   global env debug moduletype modulename vmeserver vmeaddress vsn subaddr numchans group item ipg iincr icomment iindex
   global modulechoice numchoice 
   global LUTpresent

     set module [lsearch $modulechoice $moduletype]
     if {$module == -1} then {insert-debug "cannot find $moduletype"}
     :modify MODULE :value $module

   set lvsn [lsearch $numchoice $vsn]
   set lsa [lsearch $numchoice $subaddr]
   set lchans [lsearch $numchoice $numchans]
   set litem [lsearch $numchoice $item]
   set lipg [lsearch $numchoice $ipg]
   set liincr [lsearch $numchoice $iincr]
   set liindex [lsearch $numchoice $iindex]

     :modify VSN :value $lvsn
     :modify INDEX :value $liindex
     :modify A16ADDRESS :value [first [split $vmeaddress :]]
     :modify A24ADDRESS :value [second [split $vmeaddress :]]
     :modify A32ADDRESS :value [third [split $vmeaddress :]]
     :modify ILEVEL  :value [fourth [split $vmeaddress :]]
     :modify IVECTOR :value [fifth [split $vmeaddress :]]
if {$LUTpresent != 0} {
     :modify SUBADDR :value $lsa
     :modify CHANNELS :value $lchans
     :modify GROUP :value $group
     :modify ITEM :value $litem
     :modify IPG :value $lipg
     :modify IINCR :value $liincr
}
     :modify COMMENT :value $icomment
}

proc repaint-selection {} {
  global selection
    :modify CONFIG-DATALIST :select $selection
}

proc click-add {} {
   global datalist moduletype modulename vmeserver vmeaddress vsn subaddr numchans group item ipg iincr icomment iindex
   lappend datalist  [list $moduletype $modulename $vmeserver $vmeaddress $vsn $subaddr $numchans $group $item $ipg $iincr $icomment $iindex]
   make-datalist
   clear-footer
}

proc click-replace {} {
   global selection 
   global datalist moduletype modulename vmeserver vmeaddress vsn subaddr numchans group item ipg iincr icomment iindex
   if {$selection<0} then \
      {set-footer "you haven't selected a line to replace!"; return}
   set datalist [lreplace $datalist $selection $selection \
          [list $moduletype $modulename $vmeserver $vmeaddress $vsn $subaddr $numchans $group $item $ipg $iincr $icomment $iindex]]
   make-datalist
   clear-footer
}

proc click-default {} {
   global defaultlist datalist
#    this copies the list "defaultlist" into the format of the list "datalist"
   set datalist $defaultlist
   make-datalist
}

proc click-delete {} {
   global selection datalist
   if {$selection<0} then \
      {set-footer "you haven't selected a line to delete!"; return}
   set datalist [lreplace $datalist $selection $selection]
   set selection -1
   make-datalist
   clear-footer
}

proc click-clear {} {
   global datalist
   set datalist {}
   paint-datalist
}

proc click-redisplay {} {
   global env debug
   if {$debug} then {insert-debug "click-redisplay"}
   paint-datalist
   repaint
}

proc click-printconfig {} {
  global env debug datalist

  set info "[format "%16s %8s %36s %6s %6s"  \
          channel server  VME_address   group   item]\n\n" 

  foreach data $datalist {
    set modulename [second $data]
    set vmeserver [third $data]
    set vmeaddress [fourth $data]
    set vsn [fifth $data]
    set subaddr [sixth $data]
    set numchans [seventh $data]
    set group [eigth $data]
    set item [nineth $data]
    set ipg [tenth $data]
    set iincr [eleventh $data]

    set items 0
    for {set i 0} {$i < $numchans} {incr i} {
        incr items

        append info "[format "%16s %8s %36s %6s %6s"  \
                $modulename.ch$subaddr  $vmeserver  $vmeaddress  $group $item]\n"

        incr subaddr
        incr item $iincr
        if {$items >= $ipg} then {set item [nineth $data]; incr group; set items 0}
    }
  }
  global _information_font
  set _information_font fixed
  midas-information $info
}

proc click-datalist {c s} {
  global debug selection
  if {$debug} then {insert-debug "click-datalist with arguments \{$c $s\}"}
  switch $c {
    :list-select   {set selection $s; load-data $selection}
    :list-deselect {set selection -1}
  }
  repaint
  clear-footer
}

proc double-click-datalist {c s} {
  global debug selection moduletype modulename tasknames modulechoice
  if {$debug} then {insert-debug "double-click-datalist with arguments \{$c $s\}"}
  set selection $s; load-data $selection
  set taskname [lindex $tasknames [lsearch $modulechoice $moduletype]]
  set n [second [split $modulename #]]
  inform $taskname set module $n
  inform $taskname start
}

proc click-module {n} {
  global debug modulechoice modulenames moduletype modulename
  if {$debug} then {insert-debug "click-module with argument \{$n\}"}
  set moduletype [lindex $modulechoice $n]
  set modulename [lindex $modulenames $n]
}

proc click-vsn {n} {
  global numchoice vsn
  set vsn [lindex $numchoice $n]
}

proc click-index {n} {
   global numchoice iindex
   set iindex [lindex $numchoice $n]
}

proc click-sa {n} {
  global numchoice subaddr
  set subaddr [lindex $numchoice $n]
}

proc click-channels {n} {
  global numchoice numchans
  set numchans [lindex $numchoice $n]
}

proc click-item {n} {
  global numchoice item
  set item [lindex $numchoice $n]
}

proc click-ipg {n} {
  global numchoice ipg
  set ipg [lindex $numchoice $n]
}

proc click-iincr {n} {
  global numchoice iincr
  set iincr [lindex $numchoice $n]
}
 
proc double-click arg {
   global debug
   if {$debug} then {insert-debug "double-click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   switch $w {

      CONFIG-DATALIST {double-click-datalist $c [lindex $arg 4]}
	
      default   {puts stdout $arg}
   }
}

proc start args {
   global  started
   global  VMEServerList

   if {$started == 0} then {
       load_preferences
       make-frame
       make-serverlist
       foreach server $VMEServerList {SelectServer $server}
   }
   open-frame
   initialise
   click-redisplay
}

proc autostart args {
   global  started
   if {$started == 0} then {start; close-frame} else {continue}
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}


proc click arg {
   global debug group icomment
   global A16Address A24Address A32Address Ilevel Ivector
   global LUTpresent
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
#   insert-debug "click with argument \{$arg\}"
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

	MODULE   	{click-module $v}
        INDEX           {click-index $v}
	VMESERVER	{SelectServer $i}
	A16ADDRESS	{set A16Address $v; make_vmeaddress}
	A24ADDRESS	{set A24Address $v; make_vmeaddress}
	A32ADDRESS	{set A32Address $v; make_vmeaddress}
	ILEVEL      	{set Ilevel $v; make_vmeaddress}
	IVECTOR   	{set Ivector $v; make_vmeaddress}
	VSN             {click-vsn $v}
        SUBADDR         {click-sa $v}
        CHANNELS        {click-channels $v}
	GROUP           {set group $v}
	ITEM	        {click-item $v}
	IPG	        {click-ipg $v}
        IINCR           {click-iincr $v}
        COMMENT         {set icomment $v}

	ADD	{click-add}
	REPLACE	{click-replace}
	DELETE	{click-delete}
        CLEAR   {click-clear}
        DEFAULT {click-default}

        LOAD	{if {$LUTpresent != 0} then {load-configuration+LUT} else {load-configuration}}
        LUT     {makeLUT; makeVAT}

        SAVERESTORE    {inform vmesaverestore SSRstart}
        SR             {inform vmesaverestore CSRstart}
        PRINT   {click-printconfig}

        RESET   {DAQ-reset}
        GO      {DAQ-go}
        HALT    {DAQ-halt}
        SETUP   {DAQ-setup}

	CONFIG-DATALIST {click-datalist $c $i}
	
        REDISPLAY	{click-redisplay}

        default   {puts stdout $arg}
   }
}

if {![info exists numchoice]} then {
   set numchoice {}
   for {set i 1} {$i <= 50} {incr i} {lappend numchoice $i}
}

proc make-frame {} {
   global env started
   global modulechoice
   global A16Address A24Address A32Address Ilevel Ivector
   global numchoice
   global LUTpresent

  set started 1

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vme_crate" \
      :icon-name "[self icon-title]"

   :panel CONFIGURATION  :layout T :padx 10 :pady 10
     :layout V; :row-gap 10; :col-gap 10
      :choice-stack VMESERVER :label "VME Server"
if {$LUTpresent != 0} then {set cols 144} else {set cols 100}
      :list M               :rows  2 :cols $cols :deaf T      :strings [list ""]  :scroll N
      :list CONFIG-DATALIST :rows 12 :cols $cols :exclusive T :strings [list ""] :double-click T

   :panel OPTIONS :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 15
      :choice-stack MODULE   :label "Module Type:"  :strings "$modulechoice"  :value 0
      :choice-stack INDEX    :label "Index:" :strings "$numchoice"  :value 0
      :choice-stack VSN      :label "Module ID"     :strings "$numchoice"     :value 0
      :text COMMENT :label Comment:   :w 24
      :next-row T
      :message VMEM :label "VME address"
      :text A16ADDRESS :label "A16" :value $A16Address :w 6
      :text A24ADDRESS :label "A24" :value $A24Address :w 8
      :text A32ADDRESS :label "A32" :value $A32Address :w 10
      :message VMEI :label "VME Interrupt"
      :text ILEVEL     :label "level" :value $Ilevel :w 1
      :text IVECTOR    :label "vector" :value $Ivector :w 3

if {$LUTpresent != 0} {
      :next-row T
      :choice-stack SUBADDR  :label "Channel ID:"   :exclusive T  :strings "$numchoice"       :value 0
      :choice-stack CHANNELS :label Channels:       :exclusive T  :strings "$numchoice"       :value 0
      :text GROUP         :label Group:      :w 5  
      :choice-stack ITEM  :label Item:              :exclusive T  :strings "$numchoice"    :value 0
      :choice-stack IPG   :label "Items per group:" :exclusive T  :strings "$numchoice"    :value 2
      :choice-stack IINCR :label "Item increment:"  :exclusive T  :strings "$numchoice"    :value 2
}
 
   :panel CLICK  :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 15
     :vskip 10
      :button ADD     :label Add
      :button REPLACE :label Replace
      :button DELETE  :label Delete
      :button CLEAR   :label Clear
      :button DEFAULT :label "Default Configuration"
      :button LOAD    :label "Load Configuration into Register Server"
if {$LUTpresent != 0} {
      :button LUT     :label "Generate LUT & VAT files"
      :button PRINT   :label "Print Configuration"
}

      :next-row T
      :button RESET   :label "RESET electronics"
      :button GO      :label "GO electronics"
      :button HALT    :label "HALT electronics"
      :button SETUP   :label "SETUP electronics"
if {$LUTpresent == 0} {:next-row T}
      :button SAVERESTORE    :label "Save/Restore Setup"
      :button SR             :label "Save/Restore Configuration"

   :show T

}

